local ADDON_NAME, Core = ...

--<
local format = string.format
local utils = Core.utils
local UI = Core.UI
-->

local frameCache = {}
Core.orderedTabs = {}


Core.clientTextures = {
	["App"] =  [[Interface\CHATFRAME\UI-ChatIcon-Battlenet]],
	["D3"] =   [[Interface\CHATFRAME\UI-ChatIcon-D3]],
	["Hero"] = [[Interface\CHATFRAME\UI-ChatIcon-HotS]],
	["S1"] =   [[Interface\CHATFRAME\UI-ChatIcon-SC]],
	["S2"] =   [[Interface\CHATFRAME\UI-ChatIcon-SC2]],
	["WoW"] =  [[Interface\CHATFRAME\UI-ChatIcon-WoW]],
	["WTCG"] = [[Interface\CHATFRAME\UI-ChatIcon-WTCG]],
	["Pro"] =  [[Interface\CHATFRAME\UI-ChatIcon-Overwatch]],
	["VIPR"] = [[Interface\CHATFRAME\UI-ChatIcon-CallOfDutyBlackOps4]],
	["W3"] =   [[Interface\CHATFRAME\UI-ChatIcon-Warcraft3Reforged]],
	["ODIN"] = [[Interface\CHATFRAME\UI-ChatIcon-CallOfDutyMWicon]],
	["LAZR"] = [[Interface\CHATFRAME\UI-ChatIcon-CallOfDutyMW2icon]],
}


Core.clientStrings = {
	["App"] =  "In App",
	["D3"] =   "Playing D3",
	["Hero"] = "Playing HotS",
	["S1"] =   "Playing SC1",
	["S2"] =   "Playing SC2",
	["WTCG"] = "In Hearthstone",
	["Pro"] =  "Playing OW",
	["VIPR"] = "Playing CoD:BO4",
	["W3"] =   "Playing W3:R",
	["ODIN"] = "Playing CoD:MW",
	["LAZR"] = "Playing CoD:MW2",
}


function Core.AppendTab (f)
	if #Core.orderedTabs == 0 then
		f:SetPoint("TOP")
	else 
		f:SetPoint("TOP", 0, -(#Core.orderedTabs * Core.TAB_TOTALHEIGHT))
	end
	
	if not f:IsShown() then f:Show() end
end


function Core.Tab_ReanchorAll ()
	if #Core.orderedTabs >= 1 then
		for i, t in ipairs(Core.orderedTabs) do
			t:ClearAllPoints()
			
			if i == 1 then
				t:SetPoint("TOP")
				
			else
				t:SetPoint("TOP", 0, -(i-1) * Core.TAB_TOTALHEIGHT)
			end
			
			t.tabIndex = i
		end
	end
	
	UI.RefreshTabContainer()
end


function Core.CloseTab (f)
	for i, tab in ipairs(Core.orderedTabs) do
		if tab == f then
			table.remove(Core.orderedTabs, i)
			break
		end
	end
	
	f:ClearAllPoints()
	f:Hide()
	f.tabIndex = 0
	f.portrait:SetTexCoord(0, 1, 0, 1)
	f.portrait:SetVertexColor(1, 1, 1)
	
	table.insert(frameCache, f)	
	Core.Tab_ReanchorAll()
end


function Core.BNetDisplayUpdate (tab, charName, client, isOnline)
	local fallback = isOnline and "|cFFff794dAway|r" or "|cFFb3b3b3Offline|r"

	tab.portrait:SetTexture(Core.clientTextures[client] or Core.clientTextures["App"])
	tab.infoString:SetText(charName or Core.clientStrings[client] or fallback)
end


-------------------------------------------------------------------
local function acquireTabFrame ()
-------------------------------------------------------------------
	if #frameCache > 0 then
		return table.remove(frameCache)
	end

	local frame = CreateFrame("Frame", nil, UI.tabContainer, "SessionsTabTemplate")
	frame:SetBackdrop({
		edgeFile=[[Interface\Tooltips\UI-Tooltip-Border]],
		edgeSize=8,
	})

	SetPortraitToTexture(frame.notification.BG, [[Interface\BUTTONS\RedGrad64]])
	frame.notification.BG:SetVertexColor(1,0,0)

	return frame
end


function Core.Tab_ApplyTemplate (frame, template, info, session)

	if template == "WHISPER" then
		local gender = (info[5] == 2 and "Male") or (info[5] == 3 and "Female")
		local _texture = format([[Interface\CHARACTERFRAME\TemporaryPortrait-%s-%s]], gender, info[4])
		local c = ChatTypeInfo["WHISPER"]
		frame.BG:SetGradient("Horizontal", 0,0,0, c.r*0.3, c.g*0.3, c.b*0.3)
		frame.portrait:SetTexture(_texture)
		frame.sessionName:SetText(info[6])
		frame.sessionName:SetTextColor(GetClassColor(info[2]))
		frame.infoString:SetText(Core.utils.capitalize(info[2]))
		
		
	elseif template == "BN_WHISPER" then
		frame.sessionName:SetTextColor(.6, .6, .8)
		frame.portrait:SetSize(40, 40)
		frame.sessionName:SetText(info.accountName)
		
		Core.BNetDisplayUpdate(frame, 
			info.gameAccountInfo.characterName,
			info.gameAccountInfo.clientProgram,
			info.gameAccountInfo.isOnline
		)
		
		
	elseif template == "PARTY" then
		frame.portrait:SetTexture([[Interface\COMMON\icon-]].. UnitFactionGroup("player"))
		frame.sessionName:SetText(session.title)
		local c = ChatTypeInfo["PARTY"]
		frame.sessionName:SetTextColor(c.r, c.g, c.b)
		frame.infoString:SetText("")
		
		
	elseif template == "GUILD" then
		SetLargeGuildTabardTextures("player", frame.portrait)
		frame.sessionName:SetText(session.title)
		local c = ChatTypeInfo["GUILD"]
		frame.sessionName:SetTextColor(c.r, c.g, c.b)
		local gn = GetGuildInfo("player")

		if #gn >= 12 then
			gn = gn:sub(1, 12).. "..."
		end
		
		frame.infoString:SetText(format("<%s>", gn))
		
		
	elseif template == "NEARBY" then
		frame.portrait:SetTexture([[Interface\PVPFrame\Icons\PVP-Banner-Emblem-36]])--"Interface/Worldmap/UI-World-Icon"
		frame.sessionName:SetText("Nearby")
		frame.infoString:SetText("Say & Emote")
		frame.sessionName:SetTextColor(.79, .95, 0)
		frame.BG:SetGradient("Horizontal", 0,0,0, .22, .22, 0)
		

	elseif template == "TEMP" then
		frame.portrait:SetTexture([[Interface\Garrison\Portraits\FollowerPortrait_NoPortrait]])
		frame.sessionName:SetText(Ambiguate(session.title, "short"))
		frame.sessionName:SetTextColor(1, 1, 1)
		frame.infoString:SetText("...")
	end
end


-------------------------------------------------------------------
function Core.CreateTab (info, session)
-------------------------------------------------------------------	
	local frame = acquireTabFrame()

	Core.AppendTab(frame)
	frame.portrait:SetSize(48, 48)
	frame.BG:SetGradient("Horizontal", 0,0,0, .2,.2,.38)
	
	local template = (session.needsInfo and "TEMP") or (session.name == "NEARBY" and "NEARBY") or session.chatType
	Core.Tab_ApplyTemplate(frame, template, info, session)
	
	Core.orderedTabs[#Core.orderedTabs+1] = frame
	frame.tabIndex = #Core.orderedTabs
	
	UI.RefreshTabContainer()

	return frame
end


